package scheduler.model;

import java.util.LinkedList;


/**
 * The basic alarm unit for reminders and a container of an
 * arbitrary number of possible doses.
 */
public class Bin {

	private int hour, minute;
	private LinkedList<Dose> doses;

	/**
	 * Creates a new single bin representation including
	 * zero doses by default and set to 12:00AM alarm time.
	 *
	 */
	public Bin() {
		this.doses = new LinkedList<Dose>();
	}

	
	// Setter methods **/

	/**
	 * Sets the bin alarm hour to the specified hour argument.
	 *
	 * @param hour the new alarm hour for this bin
	 */
	public void setHour(int hour) {
		this.hour = hour;
	}

	/**
	 * Sets the bin alarm minute to the specified minute argument.
	 *
	 * @param minute the new alarm minute for this bin
	 */
	public void setMinute(int minute) {
		this.minute = minute;
	}


	// Getter methods **/
	
	/**
	 * Returns this bin's alarm hour.
	 *
	 * @return alarm hour for this bin
	 */
	public int getHour() {
		return hour;
	}

	/**
	 * Returns this bin's alarm minute.
	 *
	 * @return alarm minute for this bin
	 */
	public int getMinute() {
		return minute;
	}

	
	// Dose manipulation **/

	/**
	 * Adds a single dose to the bin specified by the dose argument.
	 *
	 * @param dose the dose instance to be added to this bin
	 */
	public void addDose(Dose dose) {
		doses.add(dose);
	}

	/**
	 * Removes a single dose specified by the dose argument from the bin.
	 *
	 * @param dose the dose instance to be remove from this bin
	 */
	public void removeDose(Dose dose) {
		doses.remove(dose);
	}

	/**
	 * Returns an array of doses currently in this bin.
	 *
	 * @return a {@link Dose} array
	 */
	public Dose[] listDoses() {
		int i = 0;
		Dose[] doseArray = new Dose[doses.size()];
		for (Dose d : doses) {
			doseArray[i] = d;
			i++;
		}
		return doseArray;
	}
}

	