package scheduler.model;

/**
 * Describes an amount of a particular medication to be taken at
 * a time specified by the containing bin.
 */
public class Dose {
	
	private Medication medication;
	private int amount; 		


	/**
	 * Creates a new Dose instance for the specified
	 * medication with the amount initiated to zero.
	 *
	 * @param medication the {@link Medication} to be associated with
	 *		this dose specifically
	 */
	public Dose(Medication medication) {
		this.medication = medication;
	}


	/**
	 * Sets the medication corresponding to this dose to the argument.
	 *
	 * @param medication the new medication to associate with this dose
	 */
	public void setMedication(Medication medication) {
		this.medication = medication;
	}

	/**
	 * Sets the dose amount to the specified argument with 1/10 accuracy.
	 * Smaller dose denominations than tenths would be unreasonable.
	 *
	 * @param amount the amount of medication for this dose
	 */
	public void setAmount(double amount) {
		this.amount = (int)(amount*10);
	}


	/**
	 * Returns the medication associated with this dose.
	 *
	 * @return the medication for this dose
	 */
	public Medication getMedication() {
		return medication;
	}

	/**
	 * Returns the amount of medication associated with this dose.
	 *
	 * @return the amount for this dose
	 */
	public double getAmount() {
		return ((double)amount)/10;
	}
}


	