package scheduler.model;


/**
 * Represents a named medication with four possible
 * description attributes.
 */
public class Medication {

	private String name;
	private Color color;
	private Shape shape;
	private Special special1, special2;


	/**
	 * Creates a new Medication with the specified name and
	 * no default attributes.
	 *
	 * @param name the description for this new medication
	 */
	public Medication(String name) {
		this.name = name;
		this.color = Color.NONE;
		this.shape = Shape.NONE;
		this.special1 = Special.NONE;
		this.special2 = Special.NONE;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setColor(Color color) {
		this.color = color;
	}

	public void setShape(Shape shape) {
		this.shape = shape;
	}

	public void setSpecial1(Special special) {
		this.special1 = special;
	}

	public void setSpecial2(Special special) {
		this.special2 = special;
	}

	
	public String getName() {
		return name;
	}

	public Color getColor() {
		return color;
	}

	public Shape getShape() {
		return shape;
	}

	public Special getSpecial1() {
		return special1;
	}

	public Special getSpecial2() {
		return special2;
	}


	/**
	 * Enumerated color options for the description of a medication.
	 */ 
	public enum Color {
		NONE(0),RED(1), ORANGE(2), YELLOW(3), GREEN(4), BLUE(5), PINK(6), PURPLE(7);
		private int num;
		private Color(int num) {
			this.num = num;
		}
		/**
		 * Returns the number associated with this color.
		 *
		 * @return an integer uniquely identifying the color
		 */
		public int getNum() {
			return num;
		}
		/**
		 * Returns the Color pertaining to the specified integer.
		 * Returns Color.NONE if the integer has no matches.
		 *
		 * @param i the integer representation of a color
		 * @return the Color corresponding to i
		 */
		public static Color getColorFromInt(int i) {
			switch(i) {
				case 1: return RED;
				case 2: return ORANGE;
				case 3: return YELLOW;
				case 4: return GREEN;
				case 5: return BLUE;
				case 6: return PINK;
				case 7: return PURPLE;
				default: return NONE;
			}
		}
	}

	/**
	 * Enumerated shape options for the description of a medication.
	 */
	public enum Shape {
		NONE(0),ROUND(1), CAPSULE(2), ELLIPSOID(3), TABLET(4),
		SMALL_ROUND(5), SMALL_CAPSULE(6), SMALL_ELLIPSOID(7), SMALL_TABLET(8),
		LARGE_ROUND(9), LARGE_CAPSULE(10), LARGE_ELLIPSOID(11), LARGE_TABLET(12);
		private int num;
		private Shape(int num) {
			this.num = num;
		}
		/**
		 * Returns the number associated with this shape.
		 *
		 * @return an integer uniquely identifying the shape
		 */
		public int getNum() {
			return num;
		}
		/**
		 * Returns the Shape pertaining to the specified integer.
		 * Returns Shape.NONE if the integer has no matches.
		 *
		 * @param i the integer representation of a shape
		 * @return the Shape corresponding to i
		 */
		public static Shape getShapeFromInt(int i) {
			switch (i) {	
				case 1: return ROUND;
				case 2: return CAPSULE;
				case 3: return ELLIPSOID;
				case 4: return TABLET;
				case 5: return SMALL_ROUND;
				case 6: return SMALL_CAPSULE;
				case 7: return SMALL_ELLIPSOID;
				case 8: return SMALL_TABLET;
				case 9: return LARGE_ROUND;
				case 10: return LARGE_CAPSULE;
				case 11: return LARGE_ELLIPSOID;
				case 12: return LARGE_TABLET;
				default: return NONE;	
			}
		}		
	}

	/**
	 * Enumerated special options for the description of a medication.
	 */
	public enum Special {
		NONE(0),WITH_FOOD(1), BEFORE_MEAL(2), AFTER_MEAL(3),
		WITH_WATER(4), EMPTY_STOMACH(5), BEFORE_BED(6);
		private int num;
		private Special(int num) {
			this.num = num;
		}
		/**
		 * Returns the number associated with this special.
		 *
		 * @return an integer uniquely identifying the special
		 */
		public int getNum() {
			return num;
		}
		/**
		 * Returns the Special pertaining to the specified integer.
		 * Returns Special.NONE if the integer has no matches.
		 *
		 * @param i the integer representation of a special
		 * @return the Special corresponding to i
		 */
		public static Special getSpecialFromInt(int i) {
			switch(i) {
				case 1: return WITH_FOOD;
				case 2: return BEFORE_MEAL;
				case 3: return AFTER_MEAL;
				case 4: return WITH_WATER;
				case 5: return EMPTY_STOMACH;
				case 6: return BEFORE_BED;
				default: return NONE;
			}
		}
	}
	
}
