package scheduler.model;

import java.util.Date;
import java.util.LinkedList;

/**
 * Represents a single patient with a necessary medication
 * schedule and a device maintained by periodic schedule updates.
 */
public class Patient {

	private String patientName;
	private Date lastUpdate;
	private Schedule schedule;
	// private LinkedList<Medication> medications;

	/**
	 * Creates a new patient representation with the specified
	 * name, a blank schedule for four bins, and a lastUpdate set
	 * to the zero time (Dec 31, 1969).
	 *
	 * @param patientName the name given to the new patient instance
	 */
	public Patient(String patientName) {
		this.patientName = patientName;
		this.lastUpdate = new Date(0);
		this.schedule = new Schedule(4);
		//this.medications = new LinkedList<Medication>();
		// initialize schedule??
	}


	// Setter Methods **/

	/**
	 * Sets the patient's name to the specified argument.
	 *
	 * @param patientName the new name for this patient instance
	 */
	public void setPatientName(String patientName) {
		this.patientName = patientName;
	}

	/**
	 * Sets the last device update time and date.
	 *
	 * @param lastUpdate the new update Date instance
	 */
	public void setLastUpdate(Date lastUpdate) {
		this.lastUpdate = lastUpdate;
	}
	
	/**
	 * Sets the patients schedules to a new value.
	 *
	 * @param schedule the new schedule for this patient
	 */
	public void setSchedule(Schedule schedule) {
		this.schedule = schedule;
	}


	// Getter methods **/

	/**
	 * Returns a String representation of the patient's name.
	 *
	 * @return the patient name
	 */
	public String getPatientName() {
		return patientName;
	}

	/**
	 * Returns a Date instance representing the last update data and time.
	 *
	 * @return the last update {@link Date}
	 */
	public Date getLastUpdate() {
		return lastUpdate;
	}

	/**
	 * Returns the schedule instance pertaining to this patient.
	 *
	 * @return a schedule instance for this patient
	 */
	public Schedule getSchedule() {
		return schedule;
	}



}