package scheduler.view;

import scheduler.model.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class BinPanel extends JPanel {

	private Bin bin;

	public BinPanel(Bin bin) {
		super();
		this.bin = bin;
		init();
	}

	private void init() {
		setLayout(new BorderLayout());

		// Time Panel
		JPanel timePanel = new JPanel();
		timePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		add(timePanel, BorderLayout.NORTH);
		
			timePanel.add(new JLabel("Bin Time "));

			
			// Hour
			final JTextField hourField = new JTextField("" + bin.getHour(),2);
			hourField.setHorizontalAlignment(JTextField.RIGHT);
			hourField.addKeyListener(new KeyAdapter() {
				public void keyReleased(KeyEvent e) {
					if (hourField.getText().equals("")) {
						return;
					}
					try {
						int hour = Integer.parseInt(hourField.getText());
						if (hour >= 24 || hour < 0) {
							Toolkit.getDefaultToolkit().beep();
							hourField.setText("0");
							hourField.selectAll();
							bin.setHour(0);
						} else {
							bin.setHour(hour);
						}							
					} catch (NumberFormatException f) {
						Toolkit.getDefaultToolkit().beep();
						hourField.setText("0");
						hourField.selectAll();
						bin.setHour(0);
					}
				}
			});
			hourField.addFocusListener(new FocusListener() {
				public void focusGained(FocusEvent e) {
					hourField.selectAll();
				}
				public void focusLost(FocusEvent e) {
					hourField.select(0,0);
				}
			});
					
			timePanel.add(hourField);
		
			// Time Colon
			timePanel.add(new JLabel(":"));

			// Minute
			final JTextField minuteField = new JTextField("" + bin.getMinute(),2);
			minuteField.setHorizontalAlignment(JTextField.RIGHT);
			if (bin.getMinute() < 10) {
				minuteField.setText("0" + bin.getMinute());
			}
			minuteField.addKeyListener(new KeyAdapter() {
				public void keyReleased(KeyEvent e) {
					if (minuteField.getText().equals("")) {
						return;
					}
					try {
						int min = Integer.parseInt(minuteField.getText());
						if (min >= 59 || min < 0) {
							Toolkit.getDefaultToolkit().beep();
							minuteField.setText("0");
							minuteField.selectAll();
							bin.setMinute(0);
						} else {
							bin.setMinute(min);
						}							
					} catch (NumberFormatException f) {
						Toolkit.getDefaultToolkit().beep();
						minuteField.setText("0");
						minuteField.selectAll();
						bin.setMinute(0);
					}
				}
			});
			minuteField.addFocusListener(new FocusListener() {
				public void focusGained(FocusEvent e) {
					minuteField.selectAll();
				}
				public void focusLost(FocusEvent e) {
					minuteField.select(0,0);
					if (minuteField.getText().equals("")) {
						minuteField.setText("00");
					} else {
						int min = Integer.parseInt(minuteField.getText());
						if (min < 10) {
							minuteField.setText("0" + min);
						}
					}
				}
			});
			timePanel.add(minuteField);
			timePanel.add(new JLabel("(military time)"));
		

		// Doses Panel
		final JPanel dosesPanel = new JPanel();
		dosesPanel.setLayout(new BoxLayout(dosesPanel, BoxLayout.Y_AXIS));
		final JScrollPane scrollPane = new JScrollPane(dosesPanel);
		add(scrollPane, BorderLayout.CENTER);
		
		for (Dose d : bin.listDoses()) {
			dosesPanel.add(new DosePanel(d));
		}			

		// Buttons Panel
		JPanel buttonsPanel = new JPanel();
		add(buttonsPanel, BorderLayout.SOUTH);
		
			JButton addDoseButton = new JButton("Add Dose");
			addDoseButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Dose newDose = new Dose(null);
					bin.addDose(newDose);
					dosesPanel.add(new DosePanel(newDose));
					setSize(100,100);
				}
			});
			buttonsPanel.add(addDoseButton);
		
	}
}

	