package scheduler.view;

import scheduler.model.Dose;
import scheduler.model.Medication;
import scheduler.MedTable;
import javax.swing.*;
import java.awt.*;
import javax.swing.event.*;
import java.awt.event.*;

public class DosePanel extends JPanel {

	private Dose dose;

	public DosePanel(Dose dose) {
		super();
		this.dose = dose;
		init();
	}

	private void init() {
		setLayout(new BorderLayout());
		setBorder(BorderFactory.createTitledBorder("Dose"));
		
		// Med and Amount
		JPanel medPanel = new JPanel();
		add(medPanel, BorderLayout.NORTH);
		
			medPanel.add(new JLabel("Med"));

			final JTextField medNameField = new JTextField(15);
			if (dose.getMedication() != null) {
				medNameField.setText(dose.getMedication().getName());
			}
			medPanel.add(medNameField);

			medPanel.add(new JLabel("Amt"));
			
			final JSpinner amountSpinner = new JSpinner(new SpinnerNumberModel(0.0, 0.0, 9.5, 0.5));
			amountSpinner.setValue(dose.getAmount());
			amountSpinner.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					dose.setAmount((Double)amountSpinner.getValue());
				}
			});
			medPanel.add(amountSpinner);
		


		// Med Info
		JPanel infoPanel = new JPanel();
		infoPanel.setLayout(new GridLayout(3,4));
		add(infoPanel,BorderLayout.CENTER);
		
			infoPanel.add(new JLabel("Status:"));
			final JLabel statusLabel = new JLabel("UNKNOWN");
			infoPanel.add(statusLabel);

			infoPanel.add(new JLabel(""));
			infoPanel.add(new JLabel(""));

			infoPanel.add(new JLabel("Shape:"));
			final JLabel shapeLabel = new JLabel("");
			infoPanel.add(shapeLabel);

			infoPanel.add(new JLabel("Special#1: "));
			final JLabel special1Label = new JLabel("");
			infoPanel.add(special1Label);

			infoPanel.add(new JLabel("Color:"));
			final JLabel colorLabel = new JLabel("");
			infoPanel.add(colorLabel);

			infoPanel.add(new JLabel("Special#2: "));
			final JLabel special2Label = new JLabel("");
			infoPanel.add(special2Label);

			updateLabels(dose.getMedication(), statusLabel, colorLabel, shapeLabel, special1Label, special2Label);
		

		// Medication name listener
		medNameField.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent e) {
				if (e.getKeyChar() > 255) {
					return;
				}
				Medication med = null;
				if (!medNameField.getText().equals("") && e.getKeyCode() != KeyEvent.VK_BACK_SPACE) {
					med = MedTable.getClosestMedication(medNameField.getText());
				}
				dose.setMedication(med);
				updateLabels(med, statusLabel, colorLabel, shapeLabel, special1Label, special2Label);
				if (med != null) {
					int length = medNameField.getText().length();
					medNameField.setText(med.getName());
					medNameField.select(length, med.getName().length());
				}
			}
		});
		medNameField.addFocusListener(new FocusListener() {
			public void focusGained(FocusEvent e) {
				medNameField.selectAll();
			}
			public void focusLost(FocusEvent e) {
				medNameField.select(0,0);
			}
		});


	}

	private void updateLabels(Medication med, JLabel status, JLabel color, JLabel shape, JLabel special1, JLabel special2) {
		if (med == null) {
			status.setText("UNKNOWN");
			shape.setText("");
			color.setText("");
			special1.setText("");
			special2.setText("");
		} else {
			status.setText("KNOWN");
			shape.setText(med.getShape().toString().toLowerCase().replace('_',' '));
			color.setText(med.getColor().toString().toLowerCase().replace('_',' '));
			special1.setText(med.getSpecial1().toString().toLowerCase().replace('_',' '));
			special2.setText(med.getSpecial2().toString().toLowerCase().replace('_',' '));
		}
	}
}
			