package scheduler.view;

import scheduler.model.Patient;
import scheduler.PatientIO;
import scheduler.ProgressListener;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class PatientPanel extends JPanel {

	private Patient patient;

	public PatientPanel(Patient patient) {
		super();
		this.patient = patient;
		init();
	}

	private void init() {
		setLayout(new BorderLayout());

		// Patient Info
		final JPanel patientInfoPanel = new JPanel();
		patientInfoPanel.setLayout(new BoxLayout(patientInfoPanel, BoxLayout.Y_AXIS));
		patientInfoPanel.setBorder(BorderFactory.createTitledBorder("Patient"));
		add(patientInfoPanel, BorderLayout.NORTH);

		{
			// Patient name
			JPanel namePanel = new JPanel();
			patientInfoPanel.add(namePanel);
			//namePanel.add(new JLabel("Name: "));
			final JTextField nameField = new JTextField(patient.getPatientName(), 25);
			nameField.addKeyListener(new KeyAdapter() {
				public void keyReleased(KeyEvent e) {
					patient.setPatientName(nameField.getText());
				}
			});
			namePanel.add(nameField);

			// Last Update
			JPanel updatePanel = new JPanel();
			patientInfoPanel.add(updatePanel);
			updatePanel.add(new JLabel("Last update:"));
			final JLabel lastUpdateLabel = new JLabel(patient.getLastUpdate().toString());
			updatePanel.add(lastUpdateLabel);

			// Serial I/O Initializer
			JPanel buttonPanel = new JPanel();
			patientInfoPanel.add(buttonPanel);

			
			// Progress Listener
			final JDialog progress = new JDialog(new JDialog(),"Connection Progress");
			progress.setLocationRelativeTo(patientInfoPanel);
			progress.setLayout(new BorderLayout());
				// Label
				final JLabel progressLabel = new JLabel("\t\t\t\t");
				JPanel labelPanel = new JPanel();
				labelPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
				labelPanel.add(progressLabel);
				progress.add(labelPanel, BorderLayout.NORTH);
				// Progress Bar
				JPanel barPanel = new JPanel();
				final JProgressBar progressBar = new JProgressBar(0, 100);
				progressBar.setPreferredSize(new Dimension(250, 12));
				barPanel.add(progressBar);
			progress.add(barPanel, BorderLayout.CENTER);
			//progress.setPreferredSize(new Dimension (300, 75));
			progress.pack();

			final ProgressListener pl = new ProgressListener() {
				public void progressUpdate(String message, int val) {
					progressLabel.setText(message);
					progressBar.setValue(val);
				}
			};

			final JButton verifyButton = new JButton("Verify");
			verifyButton.setToolTipText("Identify differences with device");
			verifyButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new Thread(new Runnable() { public void run() {

					progress.setVisible(true);
					if (!PatientIO.verifyPatient(patient, pl)) {
						JOptionPane.showMessageDialog(patientInfoPanel, "Device and schedule do not match.",
							"Data Mismatch",  JOptionPane.ERROR_MESSAGE);
					} else {
						JOptionPane.showMessageDialog(patientInfoPanel, "Device and schedule match.",
							"Data Verification", JOptionPane.INFORMATION_MESSAGE);
					}
					progress.setVisible(false);
					
					}}).start();						
				}
			});
			buttonPanel.add(verifyButton);
			

			final JButton downloadButton = new JButton("Download");
			downloadButton.setToolTipText("Download patient schedule to device");
			downloadButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					new Thread(new Runnable() { public void run() {

					try {
						progress.setVisible(true);
						if(!PatientIO.downloadPatient(patient, pl)) {
							JOptionPane.showMessageDialog(patientInfoPanel, "Download failed, please retry.",
								"Serial Error",  JOptionPane.ERROR_MESSAGE);
						} else {
							JOptionPane.showMessageDialog(patientInfoPanel, "Download successful & verified",
								"Download complete", JOptionPane.INFORMATION_MESSAGE);
						}
						progress.setVisible(false);	
					} catch (Exception f) {
						JOptionPane.showMessageDialog(patientInfoPanel, "Download failed, please retry.",
								"Serial Error",  JOptionPane.ERROR_MESSAGE);
						f.printStackTrace();
					}

					}}).start();
				}
			});
			buttonPanel.add(downloadButton);
		}

		// Bins Area
		JPanel schedulePanel = new JPanel();
		JTabbedPane bins = new JTabbedPane();
		schedulePanel.setBorder(BorderFactory.createTitledBorder("Schedule"));
		schedulePanel.setLayout(new BorderLayout());
		schedulePanel.add(bins, BorderLayout.CENTER);
		add(schedulePanel, BorderLayout.CENTER);

		for (int i = 1; i < 5; i++) {
			if (patient.getSchedule().getBin(i) != null) {
				bins.addTab("Bin " + i, new BinPanel(patient.getSchedule().getBin(i)));
			}
		}			
			
	}
}